@@ -40,6 +40,7 @@ public:
     QIcon icon;
     bool ignoreShowAndResizeEventDoingAnimatedShow = false;
     KMessageWidget::MessageType messageType;
+    KMessageWidget::Position position = KMessageWidget::Inline;
     bool wordWrap;
     QList<QToolButton *> buttons;
 
@@ -89,10 +90,6 @@ void KMessageWidgetPrivate::init(KMessageWidget *q_ptr)
     closeButton->setDefaultAction(closeAction);
 
     q->setMessageType(KMessageWidget::Information);
-
-    q->connect(qApp, &QApplication::paletteChanged, q, [this] {
-        KMessageWidgetPrivate::setPalette();
-    });
 }
 
 void KMessageWidgetPrivate::createLayout()
@@ -259,6 +256,16 @@ void KMessageWidget::setText(const QString &text)
     updateGeometry();
 }
 
+Qt::TextFormat KMessageWidget::textFormat() const
+{
+    return d->textLabel->textFormat();
+}
+
+void KMessageWidget::setTextFormat(Qt::TextFormat textFormat)
+{
+    d->textLabel->setTextFormat(textFormat);
+}
+
 KMessageWidget::MessageType KMessageWidget::messageType() const
 {
     return d->messageType;
@@ -286,10 +293,26 @@ bool KMessageWidget::event(QEvent *event)
 {
     if (event->type() == QEvent::Polish && !layout()) {
         d->createLayout();
-    } else if ((event->type() == QEvent::Show && !d->ignoreShowAndResizeEventDoingAnimatedShow) || (event->type() == QEvent::LayoutRequest)) {
+    } else if ((event->type() == QEvent::Show && !d->ignoreShowAndResizeEventDoingAnimatedShow)
+               || (event->type() == QEvent::LayoutRequest && d->timeLine->state() == QTimeLine::NotRunning)) {
         setFixedHeight(d->bestContentHeight());
+
+        // if we are displaying this when application first starts, there's
+        // a possibility that the layout is not properly updated with the
+        // rest of the application because the setFixedHeight call above has
+        // the same height that was set beforehand, when we lacked a parent
+        // and thus, the layout() geometry is bogus. so we pass a bogus
+        // value to it, just to trigger a recalculation, and revert to the
+        // best content height.
+        if (geometry().height() < layout()->geometry().height()) {
+            setFixedHeight(d->bestContentHeight() + 2); // this triggers a recalculation.
+            setFixedHeight(d->bestContentHeight()); // this actually sets the correct values.
+        }
+
     } else if (event->type() == QEvent::ParentChange) {
         d->setPalette();
+    } else if (event->type() == QEvent::ApplicationPaletteChange) {
+        d->setPalette();
     }
     return QFrame::event(event);
 }
@@ -323,10 +346,25 @@ void KMessageWidget::paintEvent(QPaintEvent *event)
     const int newRed = (color.red() * alpha) + (parentWindowColor.red() * (1 - alpha));
     const int newGreen = (color.green() * alpha) + (parentWindowColor.green() * (1 - alpha));
     const int newBlue = (color.blue() * alpha) + (parentWindowColor.blue() * (1 - alpha));
-    painter.setPen(QPen(color, borderSize));
-    painter.setBrush(QColor(newRed, newGreen, newBlue));
+
     painter.setRenderHint(QPainter::Antialiasing);
-    painter.drawRoundedRect(innerRect, radius, radius);
+    painter.setBrush(QColor(newRed, newGreen, newBlue));
+    if (d->position == Position::Inline) {
+        painter.setPen(QPen(color, borderSize));
+        painter.drawRoundedRect(innerRect, radius, radius);
+        return;
+    }
+
+    painter.setPen(QPen(Qt::NoPen));
+    painter.drawRect(rect());
+
+    if (d->position == Position::Header) {
+        painter.setPen(QPen(color, 1));
+        painter.drawLine(0, rect().height(), rect().width(), rect().height());
+    } else {
+        painter.setPen(QPen(color, 1));
+        painter.drawLine(0, 0, rect().width(), 0);
+    }
 }
 
 bool KMessageWidget::wordWrap() const
@@ -350,6 +388,17 @@ void KMessageWidget::setWordWrap(bool wordWrap)
     }
 }
 
+KMessageWidget::Position KMessageWidget::position() const
+{
+    return d->position;
+}
+
+void KMessageWidget::setPosition(KMessageWidget::Position position)
+{
+    d->position = position;
+    updateGeometry();
+}
+
 bool KMessageWidget::isCloseButtonVisible() const
 {
     return d->closeButton->isVisible();
