@@ -2,15 +2,16 @@
     SPDX-FileCopyrightText: 2017 Montel Laurent <montel@kde.org>
     SPDX-FileCopyrightText: 2015 Elvis Angelaccio <elvis.angelaccio@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.0-only OR LGPL-3.0-only or LicenseRef-KDE-Accepted-LGPL
+    SPDX-License-Identifier: LGPL-2.0-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
 #include "kpasswordlineedit.h"
 
-#include <QIcon>
 #include <QAction>
+#include <QContextMenuEvent>
 #include <QHBoxLayout>
-
+#include <QIcon>
+#include <QMenu>
 
 class KPasswordLineEditPrivate
 {
@@ -20,8 +21,9 @@ public:
     {
     }
     void initialize();
-    void _k_echoModeToggled();
-    void _k_showToggleEchoModeAction(const QString &text);
+    void echoModeToggled();
+    void showToggleEchoModeAction(const QString &text);
+    void slotCustomContextMenuRequested(const QPoint &pos);
 
     QIcon passwordIcon;
     QIcon visibleIcon;
@@ -29,7 +31,7 @@ public:
     QLineEdit *passwordLineEdit = nullptr;
     QAction *toggleEchoModeAction = nullptr;
     bool isToggleEchoModeAvailable = true;
-    bool revealPasswordAvailable = true;
+    KPassword::RevealMode revealPasswordMode = KPassword::RevealMode::OnlyNew;
     KPasswordLineEdit *const q;
 };
 
@@ -40,20 +42,24 @@ void KPasswordLineEditPrivate::initialize()
     toggleEchoModeAction->setObjectName(QStringLiteral("visibilityAction"));
     toggleEchoModeAction->setVisible(false);
     toggleEchoModeAction->setToolTip(QObject::tr("Change the visibility of the password", "@info:tooltip"));
-    q->connect(toggleEchoModeAction, &QAction::triggered, q, [this]() {_k_echoModeToggled();});
-    q->connect(passwordLineEdit, &QLineEdit::textChanged, q, [this](const QString &str) {_k_showToggleEchoModeAction(str);});
+    q->connect(toggleEchoModeAction, &QAction::triggered, q, [this]() {
+        echoModeToggled();
+    });
+    q->connect(passwordLineEdit, &QLineEdit::textChanged, q, [this](const QString &str) {
+        showToggleEchoModeAction(str);
+    });
 }
 
-void KPasswordLineEditPrivate::_k_showToggleEchoModeAction(const QString &text)
+void KPasswordLineEditPrivate::showToggleEchoModeAction(const QString &text)
 {
-    if (revealPasswordAvailable) {
+    if (revealPasswordMode != KPassword::RevealMode::Never) {
         toggleEchoModeAction->setVisible(isToggleEchoModeAvailable && (passwordLineEdit->echoMode() == QLineEdit::Normal || !text.isEmpty()));
     } else {
         toggleEchoModeAction->setVisible(false);
     }
 }
 
-void KPasswordLineEditPrivate::_k_echoModeToggled()
+void KPasswordLineEditPrivate::echoModeToggled()
 {
     if (passwordLineEdit->echoMode() == QLineEdit::Password) {
         passwordLineEdit->setEchoMode(QLineEdit::Normal);
@@ -71,6 +77,21 @@ void KPasswordLineEditPrivate::_k_echoModeToggled()
     Q_EMIT q->echoModeChanged(passwordLineEdit->echoMode());
 }
 
+void KPasswordLineEditPrivate::slotCustomContextMenuRequested(const QPoint &pos)
+{
+    auto popup = std::unique_ptr<QMenu>(passwordLineEdit->createStandardContextMenu());
+    if (popup) {
+        if (toggleEchoModeAction->isVisible()) {
+            popup->addSeparator();
+
+            toggleEchoModeAction->setText(passwordLineEdit->echoMode() == QLineEdit::Password ? QObject::tr("Show Password", "@action:inmenu")
+                                                                                              : QObject::tr("Hide Password", "@action:inmenu"));
+            popup->addAction(toggleEchoModeAction);
+        }
+        popup->exec(pos);
+    }
+}
+
 KPasswordLineEdit::KPasswordLineEdit(QWidget *parent)
     : QWidget(parent)
     , d(new KPasswordLineEditPrivate(this))
@@ -81,6 +102,10 @@ KPasswordLineEdit::KPasswordLineEdit(QWidget *parent)
     d->passwordLineEdit = new QLineEdit(this);
     d->passwordLineEdit->setObjectName(QStringLiteral("passwordlineedit"));
     d->passwordLineEdit->setEchoMode(QLineEdit::Password);
+    d->passwordLineEdit->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(d->passwordLineEdit, &QWidget::customContextMenuRequested, this, [this](const QPoint &pos) {
+        d->slotCustomContextMenuRequested(pos);
+    });
     connect(d->passwordLineEdit, &QLineEdit::textChanged, this, &KPasswordLineEdit::passwordChanged);
     setFocusProxy(d->passwordLineEdit);
     setFocusPolicy(d->passwordLineEdit->focusPolicy());
@@ -88,15 +113,14 @@ KPasswordLineEdit::KPasswordLineEdit(QWidget *parent)
     d->initialize();
 }
 
-KPasswordLineEdit::~KPasswordLineEdit()
-{
-    delete d;
-}
+KPasswordLineEdit::~KPasswordLineEdit() = default;
 
 void KPasswordLineEdit::setPassword(const QString &password)
 {
     if (d->passwordLineEdit->text() != password) {
-        d->isToggleEchoModeAvailable = password.isEmpty();
+        if (d->revealPasswordMode == KPassword::RevealMode::OnlyNew) {
+            d->isToggleEchoModeAvailable = password.isEmpty();
+        }
         d->passwordLineEdit->setText(password);
     }
 }
@@ -131,20 +155,43 @@ QLineEdit::EchoMode KPasswordLineEdit::echoMode() const
     return d->passwordLineEdit->echoMode();
 }
 
+void KPasswordLineEdit::setReadOnly(bool readOnly)
+{
+    d->passwordLineEdit->setReadOnly(readOnly);
+}
+
+bool KPasswordLineEdit::isReadOnly() const
+{
+    return d->passwordLineEdit->isReadOnly();
+}
+
 QLineEdit *KPasswordLineEdit::lineEdit() const
 {
     return d->passwordLineEdit;
 }
 
+#if KWIDGETSADDONS_ENABLE_DEPRECATED_SINCE(6, 0)
 void KPasswordLineEdit::setRevealPasswordAvailable(bool reveal)
 {
-    d->revealPasswordAvailable = reveal;
-    d->_k_showToggleEchoModeAction(password());
+    d->revealPasswordMode = reveal ? KPassword::RevealMode::OnlyNew : KPassword::RevealMode::Never;
+    d->showToggleEchoModeAction(password());
 }
 
 bool KPasswordLineEdit::isRevealPasswordAvailable() const
 {
-    return d->revealPasswordAvailable;
+    return d->revealPasswordMode != KPassword::RevealMode::Never;
+}
+#endif
+
+void KPasswordLineEdit::setRevealPasswordMode(KPassword::RevealMode mode)
+{
+    d->revealPasswordMode = mode;
+    d->showToggleEchoModeAction(password());
+}
+
+KPassword::RevealMode KPasswordLineEdit::revealPasswordMode() const
+{
+    return d->revealPasswordMode;
 }
 
 QAction *KPasswordLineEdit::toggleEchoModeAction() const
